'use client';

import { useEffect, useState } from 'react';
import {
  Connection,
  PublicKey,
  Transaction,
  SystemProgram,
} from '@solana/web3.js';
import axios from 'axios';
import { usePrivy } from '@privy-io/react-auth';
import {
  useSolanaWallets,
  useSignTransaction,
} from '@privy-io/react-auth/solana';

interface GpuPaymentModalProps {
  isOpen: boolean;
  onClose: () => void;
  gpu: {
    id: string;
    title: string;
    price_usd: number;
  };
}

const SOLANA_RPC = 'https://api.mainnet-beta.solana.com';
const BUSINESS_WALLET = 'Y93ednSpre2XRjPTBafHU1BXPXKMPhujcYAEshS5pXm8K'; // <-- Replace this

export const GpuPaymentModal = ({ isOpen, onClose, gpu }: GpuPaymentModalProps) => {
  const { user } = usePrivy();
  const { wallets } = useSolanaWallets();
  const { signTransaction } = useSignTransaction();

  const [solPrice, setSolPrice] = useState<number | null>(null);
  const [loading, setLoading] = useState(false);
  const [errorMsg, setErrorMsg] = useState<string | null>(null);
  const [successMsg, setSuccessMsg] = useState<string | null>(null);

  const connection = new Connection(SOLANA_RPC);

  useEffect(() => {
    const fetchSolPrice = async () => {
      try {
        const { data } = await axios.get(
          'https://api.coingecko.com/api/v3/simple/price?ids=solana&vs_currencies=usd'
        );
        setSolPrice(data.solana.usd);
      } catch {
        setErrorMsg('Failed to fetch SOL price.');
      }
    };

    if (isOpen) {
      fetchSolPrice();
      setErrorMsg(null);
      setSuccessMsg(null);
    }
  }, [isOpen]);

  const handlePayment = async () => {
    setLoading(true);
    setErrorMsg(null);
    setSuccessMsg(null);

    try {
      const solWallet = wallets[0];
      if (!solWallet || !solWallet.address) {
        throw new Error('No connected Solana wallet found.');
      }

      const fromPubKey = new PublicKey(solWallet.address);
      const toPubKey = new PublicKey(BUSINESS_WALLET);
      const solAmount = (gpu.price_usd / (solPrice || 1)).toFixed(6);

      const transaction = new Transaction().add(
        SystemProgram.transfer({
          fromPubkey: fromPubKey,
          toPubkey: toPubKey,
          lamports: Math.floor(parseFloat(solAmount) * 1e9),
        })
      );

      transaction.feePayer = fromPubKey;
      const { blockhash } = await connection.getRecentBlockhash();
      transaction.recentBlockhash = blockhash;

      const signedTx = await signTransaction({
        transaction,
        connection,
        address: solWallet.address,
      });

      const txId = await connection.sendRawTransaction(signedTx.serialize());
      await connection.confirmTransaction(txId, 'confirmed');

      await axios.post('/api/orders', {
        gpuId: gpu.id,
        userEmail: user?.email?.address,
        txSignature: txId,
        status: 'success',
      });

      setSuccessMsg(`Payment successful! Transaction ID: ${txId}`);
    } catch (err: any) {
      setErrorMsg(err.message || 'Payment failed.');
      await axios.post('/api/orders', {
        gpuId: gpu.id,
        userEmail: user?.email?.address,
        txSignature: null,
        status: 'failed',
      });
    } finally {
      setLoading(false);
    }
  };

  if (!isOpen) return null;

  const solAmount = solPrice ? (gpu.price_usd / solPrice).toFixed(6) : '...';

  return (
    <div className="fixed inset-0 z-50 flex items-center justify-center bg-black/50">
      <div className="bg-white w-full max-w-md rounded-xl p-6 shadow-xl">
        <h2 className="text-xl font-semibold text-gray-800">{gpu.title}</h2>
        <p className="text-gray-600 mt-1">
          Price: <span className="font-medium">${gpu.price_usd} USD</span> (~{solAmount} SOL)
        </p>

        {errorMsg && (
          <div className="mt-4 rounded-md bg-red-100 px-4 py-2 text-sm text-red-700">
            {errorMsg}
          </div>
        )}

        {successMsg && (
          <div className="mt-4 rounded-md bg-green-100 px-4 py-2 text-sm text-green-700">
            {successMsg}
          </div>
        )}

        <div className="mt-6 flex justify-end gap-3">
          <button
            onClick={onClose}
            disabled={loading}
            className="px-4 py-2 rounded-lg border border-gray-300 text-gray-600 hover:bg-gray-100 transition"
          >
            Cancel
          </button>
          <button
            onClick={handlePayment}
            disabled={loading}
            className="px-4 py-2 rounded-lg bg-green-600 text-white hover:bg-green-700 transition disabled:opacity-60"
          >
            {loading ? 'Processing...' : 'Confirm Payment'}
          </button>
        </div>
      </div>
    </div>
  );
};

export default GpuPaymentModal;
